# Copyright (C) 2021-2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from .appleudid import AppleUDID
from .decimalmeid import DecimalMEID
from .deviceserialnumber import DeviceSerialNumber
from .imei import IMEI
from .meid import MEID
from .privateenterprisenumber import PrivateEnterpriseNumber
from .ordernumber import OrderNumber
from .purchaseordernumber import PurchaseOrderNumber


__all__: list[str] = [
    'AppleUDID',
    'DecimalMEID',
    'DeviceSerialNumber',
    'IMEI',
    'MEID',
    'PrivateEnterpriseNumber',
    'OrderNumber',
    'PurchaseOrderNumber',
]


# Copyright (C) 2022 Cochise Ruhulessin
# 
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import os
from typing import Any


GOOGLE_DATASTORE_NAMESPACE: str = os.environ['GOOGLE_DATASTORE_NAMESPACE']

GOOGLE_SERVICE_PROJECT: str = os.environ['GOOGLE_SERVICE_PROJECT']

GLOBAL_DEPENDENCIES: list[dict[str, Any]] = [
    {
        'name': 'MessageTransport',
        'qualname': 'cbra.ext.google.GooglePubsubTransport',
        'kwargs': {
            'project': GOOGLE_SERVICE_PROJECT,
            'prefix': 'molano',
            'service': os.environ['APP_NAME']
        }
    },
]


# Copyright (C) 2021-2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import hashlib

import pydantic

from .privateenterprisenumber import PrivateEnterpriseNumber


class DeviceSerialNumber(pydantic.BaseModel):
    manufacturer: PrivateEnterpriseNumber
    model: str
    serial: str

    def sha256(self) -> str:
        h = hashlib.sha3_256()
        h.update(str.encode(type(self).__name__))
        h.update(str.encode(self.manufacturer.sha256(), encoding='ascii'))
        h.update(str.encode(self.model, encoding='utf-8'))
        h.update(str.encode(self.serial, encoding='utf-8'))
        return h.hexdigest()


# Copyright (C) 2022 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from cbra.conf import settings
from cbra.ext.google import GoogleTransport
from cbra.ext import ioc
from cbra.ext.ioc.provider import _default # type: ignore
from cbra.ext.ioc.models import DependencyList

from molano.runtime.ioc import GLOBAL_DEPENDENCIES


async def init():
    """Loads the preconfigured dependencies in the default provider."""
    await _default.load_many(DependencyList(items=GLOBAL_DEPENDENCIES)) # type: ignore
    ioc.provide(
        name='MessageTransport',
        value=GoogleTransport(
            project=settings.GOOGLE_HOST_PROJECT,
            command_topic=settings.AORTA_COMMAND_TOPIC,
            command_project=settings.AORTA_COMMAND_PROJECT,
            events_topic=settings.AORTA_EVENT_TOPIC,
            events_project=settings.AORTA_EVENT_PROJECT
        ),
        force=True
    )


# Copyright (C) 2022 Cochise Ruhulessin
# 
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
from datetime import datetime
from datetime import timezone

import aorta
import pydantic

from ..deviceinfo import DeviceInfo
from ..deviceinspector import DeviceInspector


class DeviceInspected(aorta.Event):
    id: int
    inspected: datetime = pydantic.Field(
        default_factory=lambda: datetime.now(timezone.utc)
    )
    device: DeviceInfo
    source: DeviceInspector


 # Copyright (C) 2021-2023 Cochise Ruhulessin
#
# All rights reserved. No warranty, explicit or implicit, provided. In
# no event shall the author(s) be liable for any claim or damages.
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
import canonical


NAME_MAP: dict[str, tuple[str, str]] = {
    'apple'     : ('1.3.6.1.4.1.63', 'Apple'),
    'samsung'   : ('1.3.6.1.4.1.236', 'Samsung'),
}
OID_MAP: dict[str, str] = ({x[0]: x[1] for x in NAME_MAP.values()})


class PrivateEnterpriseNumber(canonical.StringType):
    __module__: str = 'molano.canon'

    @classmethod
    def parse_name(cls, value: str) -> 'PrivateEnterpriseNumber':
        value = str.lower(value)
        if value not in NAME_MAP:
            raise LookupError(f"Unknown manufacturer: {value[:64]}")
        return cls(NAME_MAP[value][0])

    @classmethod
    def validate(cls, v: str) -> str:
        if v in NAME_MAP:
            v = NAME_MAP[v][0]
        return cls(v)
    
    @property
    def name(self) -> str:
        return OID_MAP[str(self)]